'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = getDeps;

var _readline = require('readline');

var _readline2 = _interopRequireDefault(_readline);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _globby = require('globby');

var _globby2 = _interopRequireDefault(_globby);

var _utils = require('./utils');

var _pkgError = require('./pkg-error');

var _pkgError2 = _interopRequireDefault(_pkgError);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getDeps(input, flags) {
  const files = _globby2.default.sync(['!**/node_modules/**', ...input]);
  const allDeps = [];

  files.forEach(file => {
    allDeps.push(getFileDeps((0, _utils.cwd)(file), flags));
  });
  return Promise.all(allDeps).then(results => {
    const pkgs = new Set(results.reduce((a, b) => a.concat(b), []));
    if ([...pkgs].length === 0) {
      return Promise.reject(new _pkgError2.default('no pkgs to install'));
    }
    return [...pkgs];
  });
}

function getFileDeps(file, flags) {
  const isES = flags.es;
  const isAll = flags.all;
  const deps = [];
  // require\(\W+([\w-.]+)[\/\w]*[\W]+\)
  const reg = /require\(['"]+([a-zA-z][\w-.]+)[\/\w]*['"]+\)/i; // eslint-disable-line no-useless-escape
  const regES = /import[^."]+['"]+([a-zA-z][\w-.]+)[\/\w]*['"]+/i; // eslint-disable-line no-useless-escape

  const rl = _readline2.default.createInterface({
    input: _fs2.default.createReadStream(file)
  });

  return new Promise(resolve => {
    rl.on('line', line => {
      if (!isES || isAll) {
        const matches = line.match(reg);
        if (matches) {
          deps.push(matches[1]);
        }
      }
      if (isES || isAll) {
        const matchesES = line.match(regES);
        if (matchesES) {
          deps.push(matchesES[1]);
        }
      }
    }).on('close', () => {
      return resolve(deps);
    });
  });
}